--[[
FantasyLoot
Item link modification

Author: Michael Fu "Mithorium" (Morodan @ Khadgar-US)

The code of this addon is licensed under a Creative Commons Attribution-Noncommercial-Share Alike 3.0 License.
(see license.txt) All included textures and sounds are copyrighted by their respective owners, license information
for these media files can be found in the modules that make use of them.

 You are free:
   * to Share - to copy, distribute, display, and perform the work
   * to Remix - to make derivative works
 Under the following conditions:
   * Attribution.   You must attribute the work in the manner specified by the author or licensor (but not in any way
                    that suggests that they endorse you or your use of the work).
   * Noncommercial. You may not use this work for commercial purposes.
   * Share Alike.   If you alter, transform, or build upon this work, you may distribute the resulting work only under
                    the same or similar license to this one.
]]
local _, _AG = ...
local L = _AG.L

-- /* Setup */
FantasyLoot = LibStub("AceAddon-3.0"):NewAddon("FantasyLoot", "AceConsole-3.0")

-- /* Locals */
-- Reforge Tables
local indexIndexToReforge = {
  {0, 113, 114, 115, 116, 117, 118, 119},
  {120, 0, 121, 122, 123, 124, 125, 126},
  {127, 128, 0, 129, 130, 131, 132, 133},
  {134, 135, 136, 0, 137, 138, 139, 140},
  {141, 142, 143, 144, 0, 145, 146, 147},
  {148, 149, 150, 151, 152, 0, 153, 154},
  {155, 156, 157, 158, 159, 160, 0, 161},
  {162, 163, 164, 165, 166, 167, 168, 0},
}
local REFORGE_TABLE_BASE = 112
local reforgeIndexToStatPair = {
  {1, 2}, {1, 3}, {1, 4}, {1, 5}, {1, 6}, {1, 7}, {1, 8},
  {2, 1}, {2, 3}, {2, 4}, {2, 5}, {2, 6}, {2, 7}, {2, 8},
  {3, 1}, {3, 2}, {3, 4}, {3, 5}, {3, 6}, {3, 7}, {3, 8},
  {4, 1}, {4, 2}, {4, 3}, {4, 5}, {4, 6}, {4, 7}, {4, 8},
  {5, 1}, {5, 2}, {5, 3}, {5, 4}, {5, 6}, {5, 7}, {5, 8},
  {6, 1}, {6, 2}, {6, 3}, {6, 4}, {6, 5}, {6, 7}, {6, 8},
  {7, 1}, {7, 2}, {7, 3}, {7, 4}, {7, 5}, {7, 6}, {7, 8},
  {8, 1}, {8, 2}, {8, 3}, {8, 4}, {8, 5}, {8, 6}, {8, 7},
}

_AG.upgradeByType = _AG.upgradeByType or {
  [1] = "Valor t16", [2] = "Mystery t16", [3] = "Valor t15", [4] = "Mystery t15",
  [5] = "Valor t13/14", [6] = "Mystery t13/14", [7] = "Justice", [8] = "Conquest s12", [9] = "Honor s12",
  ["Valor t16"] = {levels = 4, [1] = {start = 491, levels = 2}, [2] = {start = 504, levels = 1}},
  ["Mystery t16"] = {levels = 6, [1] = {start = 494, levels = 4}, [2] = {start = 506, levels = 1}},
  ["Valor t15"] = {start = 465, levels = 2},
  ["Mystery t15"] = {start = 468, levels = 4},
  ["Valor t13/14"] = {start = 445, levels = 2},
  ["Mystery t13/14"] = {start = 458, levels = 4},
  ["Justice"] = {start = 451, levels = 1},
  ["Conquest s12"] = {start = 453, levels = 2},
  ["Honor s12"] = {start = 456, levels = 1},
}
local upgradeByType = _AG.upgradeByType

_AG.gemIndex = _AG.gemIndex or {
  "Red", "Blue", "Yellow", "Orange", "Green", "Purple", "Meta", "Legendary", "Cogwheel"
}
local gemIndex = _AG.gemIndex

-- Default DB
FantasyLoot.defaultDB = {
  db_version = 1,
  -- savedEnchants = {
    -- [4442] = "Jade Spirit",
    -- [4444] = "Dancing Steel",
  -- },
  gemsKnown = 0,
  gemToStatsByType = {
    Red    = {},
    Blue   = {},
    Yellow = {},
    Orange = {},
    Green  = {},
    Purple = {},
    Meta   = {},
    Legendary = {},
    Cogwheel  = {},
  },
}

local ItemMeta = {}

-- Helpers

local function packageItemLink(i)
  return "|cff"..i.color.."|Hitem:"..i.id..":"..i.enchant..":"..i.gem1..":"..i.gem2..":"..i.gem3..":"..i.gem4..":"..i.suffix..":"..i.unique..":"..i.linklvl..":"..i.reforge..":"..i.upgrade.."|h["..i.name.."]|h|r"
end

local function println(text)
	DEFAULT_CHAT_FRAME:AddMessage(("|cffff7d0a[|r|cffffd200FantasyLoot|r|cffff7d0a]|r %s"):format(tostring(text)), 0.9, 1, 0.8)
end

local function printerr(text)
	DEFAULT_CHAT_FRAME:AddMessage(("|cffff7d0a[|r|cffffd200FantasyLoot|r|cffff7d0a]|r %s"):format(tostring(text)), 1, 0.2, 0.25)
end

local function InitDB(tbl, defaults)
	if type(tbl) == "string" then
		local name = tbl
		tbl = _G[name]
		if not tbl then
			tbl = defaults
			_G[name] = tbl
		end
	end

	return tbl
end

-- /* Init */
function FantasyLoot:OnInitialize()
	-- Create DB
	self.db = InitDB("FantasyLootDB", self.defaultDB)
	if not self.db then
    printerr("Error: Database corrupted. Please remove FantasyLoot.lua from your saved variables folder.")
  end
  
  self:ClearItem()
  
  self.frame = nil
	
	self:RegisterChatCommand("fantasyloot", "OnCommand")
  self:RegisterChatCommand("fl", "OnCommand")
  
  FantasyLootUI:OnInitialize()
end

-- /* API */
function FantasyLoot:OnCommand(argv)
  arg = self:GetArgs(argv) or ""
  
  if arg == "toggle" then
    if FantasyLootFrame:IsShown() then
      self:Hide()
    else
      self:Show()
    end
  elseif arg == "hide" then
    self:Hide()
  elseif tonumber(arg) ~= nil then
    self:SetItemID(arg)
    self:Show()
  else
    self:SetItem(arg)
    self:Show()
  end
end

function FantasyLoot:Show()
  if not FantasyLootFrame then
    return
  end
  if not FantasyLootFrame:IsShown() then
    FantasyLootFrame:Show()
  else
    FantasyLootUI:Update(FantasyLootFrame)
  end
end

function FantasyLoot:Hide()
  if not FantasyLootFrame then
    return
  end
  if FantasyLootFrame:IsShown() then
    FantasyLootFrame:Hide()
  end
end

function FantasyLoot:SetItemID(itemID)
  local itemLink = select(2, GetItemInfo(itemID))
  
  if itemLink == nil then
    return false
  end
  
  return self:SetItem(itemLink)
end

function FantasyLoot:SetItem(itemLink)
  local color, id, enchant, gem1, gem2, gem3, gem4, suffix, unique, linklvl, reforge, upgrade, name =
  itemLink:match("|cff(%x+)|Hitem:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%-?%d+):(%-?%d+):(%d+):(%d+):(%d+)|h%[([^%]]+)%]|h|r")
  
  if id == nil then
    return false
  end
  
  local i = self.item
  i.color = color
  i.id = id
  i.enchant = enchant
  i.gem1 = gem1
  i.gem2 = gem2
  i.gem3 = gem3
  i.gem4 = gem4
  i.suffix = suffix
  i.unique = unique
  i.linklvl = linklvl
  i.reforge = reforge
  i.upgrade = upgrade
  i.name = name
  
  -- Build meta table
  local reforgePair = self:GetItemReforgeStats()
  ItemMeta.reforgeA = (reforgePair and reforgePair[1]) or 0
  
  return true
end

function FantasyLoot:GetItem()
  if self.item.id then
    return packageItemLink(self.item)
  else
    return nil
  end
end

function FantasyLoot:ClearItem()
  self.item = {
    color = nil,
    id = nil,
    enchant = nil,
    gem1 = nil,
    gem2 = nil,
    gem3 = nil,
    gem4 = nil,
    suffix = nil,
    unique = nil,
    linklvl = nil,
    reforge = nil,
    upgrade = nil,
    name = nil,
  }
end

function FantasyLoot:GenerateGemTable()
  -- Update as nececcary for patch changes
  local GEM_START_ID = 4500
  local GEM_END_ID = 5034
  ----------------------------------------
  if not FantasyLootFrame then
    return
  end
  local gemToStatsByType = {
    Red = {},
    Blue = {},
    Yellow = {},
    Orange = {},
    Green = {},
    Purple = {},
    Meta = {},
    Legendary = {},
    Cogwheel = {},
  }
  local textureToType = {
      inv_misc_gem_x4_rare_cut_red = "Red",
      inv_misc_gem_x4_rare_cut_blue = "Blue",
      inv_misc_gem_x4_rare_cut_yellow = "Yellow",
      inv_misc_gem_x4_rare_cut_orange = "Orange",
      inv_misc_gem_x4_rare_cut_green = "Green",
      inv_misc_gem_x4_rare_cut_purple = "Purple",
      inv_jewelcrafting_dragonseye05 = "Red",
      inv_jewelcrafting_dragonseye04 = "Blue",
      inv_jewelcrafting_dragonseye03 = "Yellow",
      inv_misc_gem_x4_metagem_cut = "Meta",
      inv_legendary_breathofblackprince_int = "Legendary",
      inv_legendary_breathofblackprince_agi = "Legendary",
      inv_legendary_breathofblackprince_str = "Legendary",
      inv_legendary_chimeraoffear = "Legendary",
      inv_misc_enggizmos_30 = "Cogwheel",
  }
  local texture,stats
  local testTemplate = "|cffff8000|Hitem:80211:0:%d:0:0:0:0:0:90:0:0|h[Enchanting Test Sword]|h|r"
  local count = 0
  
  FantasyLootFrameStatusFrame.StatusText:SetText("Scanning... Please do not move your mouse.")
  
  for gem = GEM_START_ID, GEM_END_ID do
    FantasyLootScanningTooltip:ClearLines()
    FantasyLootScanningTooltip:SetHyperlink(format(testTemplate,gem))
    if FantasyLootScanningTooltipTexture1:IsShown() then
      texture = ((FantasyLootScanningTooltipTexture1:GetTexture() or ""):match("Interface\\Icons\\(.+)") or ""):lower()
      if texture and textureToType[texture] then
        stats = select(2,FantasyLootScanningTooltipTexture1:GetPoint()):GetText():match("|c%x+(%+.+)|r")
        gemToStatsByType[textureToType[texture]][gem] = {text = stats, texture = texture}
        count = count + 1
      end
    end
  end
  self.db.gemsKnown = count
  self.db.gemToStatsByType = gemToStatsByType
  
  FantasyLootFrameStatusFrame.StatusText:SetText(count..L[" gems discovered."])
end

function FantasyLoot:GetEnchantName(enchantID)
  local testTemplate = "|cffff8000|Hitem:80211:%d:0:0:0:0:0:0:90:0:0|h[Enchanting Test Sword]|h|r"
  FantasyLootScanningTooltip:ClearLines()
  FantasyLootScanningTooltip:SetHyperlink(format(testTemplate,enchantID))
  return (FantasyLootScanningTooltipTextLeft7:GetText() or ""):match(ENCHANTED_TOOLTIP_LINE:format("(.+)")) or ""
end

function FantasyLoot:GetItemReforgeStats()
  local reforgeID = tonumber(self.item.reforge)
  if reforgeID == 0 then
    return nil
  end
  local index = reforgeID - REFORGE_TABLE_BASE
  return reforgeIndexToStatPair[index]
end

function FantasyLoot:GetItemUpgrade()
  local upgradeID = tonumber(self.item.upgrade)
  if upgradeID == 0 then
    return nil
  end
  for _, k in ipairs(upgradeByType) do
    local v = upgradeByType[k]
    if v.start ~= nil then
      if upgradeID >= v.start and upgradeID <= v.start + v.levels then
        return k
      end
    else
      for _, w in ipairs(v) do
        if upgradeID >= w.start and upgradeID <= w.start + w.levels then
          return k
        end
      end
    end
  end
  return nil
end

function FantasyLoot:GetItemSocketInfo(socket)
  local gem = tonumber(self.item["gem"..socket])
  local v
  for _,k in ipairs(gemIndex) do
    v = self.db.gemToStatsByType[k]
    if v[gem] then
      return v[gem].text, "Interface\\Icons\\"..v[gem].texture
    end
  end
  return nil
end

function FantasyLoot:DeltaApply(key, value)
  if key == "reforgeA" then
    if value == 0 then
      ItemMeta.reforgeA = 0
      self.item.reforge = 0
    else
      ItemMeta.reforgeA = value
      self.item.reforge = 0
    end
  elseif key == "reforgeB" then
    -- Update reforgeID
    self.item.reforge = indexIndexToReforge[ItemMeta.reforgeA][value]
  elseif key == "upgradeT" then
    if value == 0 then
      self.item.upgrade = 0
    end
  elseif key == "upgradeID" then
    self.item.upgrade = value
  elseif key == "enchantID" then
    self.item.enchant = value
  elseif key == "gem1" then
    self.item.gem1 = value
  elseif key == "gem2" then
    self.item.gem2 = value
  elseif key == "gem3" then
    self.item.gem3 = value
  elseif key == "gem4" then
    self.item.gem4 = value
  end
end